@@ -4,7 +4,7 @@
 *
 * Perform PPPoE discovery
 *
-* Copyright (C) 1999 by Roaring Penguin Software Inc.
+* Copyright (C) 1999-2012 by Roaring Penguin Software Inc.
 *
 * LIC: GPL
 *
@@ -12,6 +12,7 @@
 
 static char const RCSID[] =
 "$Id$";
+#define _GNU_SOURCE 1
 
 #include "pppoe.h"
 
@@ -43,8 +44,13 @@ static char const RCSID[] =
 
 #include <signal.h>
 
-/* Supplied by pppd if we're a plugin */
-extern int persist;
+#ifdef PLUGIN
+#include "pppd/pppd.h"
+#include "pppd/fsm.h"
+#include "pppd/lcp.h"
+#else
+int persist = 0;
+#endif
 
 /**********************************************************************
 *%FUNCTION: parseForHostUniq
@@ -119,6 +125,9 @@ parsePADOTags(UINT16_t type, UINT16_t len, unsigned char *data,
     struct PacketCriteria *pc = (struct PacketCriteria *) extra;
     PPPoEConnection *conn = pc->conn;
     int i;
+#ifdef PLUGIN
+    UINT16_t mru;
+#endif
 
     switch(type) {
     case TAG_AC_NAME:
@@ -174,7 +183,10 @@ parsePADOTags(UINT16_t type, UINT16_t len, unsigned char *data,
 	    printf("Got a Service-Name-Error tag: %.*s\n", (int) len, data);
 	} else {
 	    pktLogErrs("PADO", type, len, data, extra);
-	    exit(1);
+	    pc->gotError = 1;
+	    if (!persist) {
+		exit(1);
+	    }
 	}
 	break;
     case TAG_AC_SYSTEM_ERROR:
@@ -182,7 +194,10 @@ parsePADOTags(UINT16_t type, UINT16_t len, unsigned char *data,
 	    printf("Got a System-Error tag: %.*s\n", (int) len, data);
 	} else {
 	    pktLogErrs("PADO", type, len, data, extra);
-	    exit(1);
+	    pc->gotError = 1;
+	    if (!persist) {
+		exit(1);
+	    }
 	}
 	break;
     case TAG_GENERIC_ERROR:
@@ -190,9 +205,25 @@ parsePADOTags(UINT16_t type, UINT16_t len, unsigned char *data,
 	    printf("Got a Generic-Error tag: %.*s\n", (int) len, data);
 	} else {
 	    pktLogErrs("PADO", type, len, data, extra);
-	    exit(1);
+	    pc->gotError = 1;
+	    if (!persist) {
+		exit(1);
+	    }
 	}
 	break;
+#ifdef PLUGIN
+    case TAG_PPP_MAX_PAYLOAD:
+	if (len == sizeof(mru)) {
+	    memcpy(&mru, data, sizeof(mru));
+	    mru = ntohs(mru);
+	    if (mru >= ETH_PPPOE_MTU) {
+		if (lcp_allowoptions[0].mru > mru) lcp_allowoptions[0].mru = mru;
+               if (lcp_wantoptions[0].mru > mru) lcp_wantoptions[0].mru = mru;
+               conn->seenMaxPayload = 1;
+	    }
+	}
+	break;
+#endif
     }
 }
 
@@ -212,6 +243,9 @@ static void
 parsePADSTags(UINT16_t type, UINT16_t len, unsigned char *data,
 	      void *extra)
 {
+#ifdef PLUGIN
+    UINT16_t mru;
+#endif
     PPPoEConnection *conn = (PPPoEConnection *) extra;
     switch(type) {
     case TAG_SERVICE_NAME:
@@ -228,6 +262,19 @@ parsePADSTags(UINT16_t type, UINT16_t len, unsigned char *data,
 	conn->relayId.length = htons(len);
 	memcpy(conn->relayId.payload, data, len);
 	break;
+#ifdef PLUGIN
+    case TAG_PPP_MAX_PAYLOAD:
+	if (len == sizeof(mru)) {
+	    memcpy(&mru, data, sizeof(mru));
+	    mru = ntohs(mru);
+	    if (mru >= ETH_PPPOE_MTU) {
+		if (lcp_allowoptions[0].mru > mru) lcp_allowoptions[0].mru = mru;
+               if (lcp_wantoptions[0].mru > mru) lcp_wantoptions[0].mru = mru;
+               conn->seenMaxPayload = 1;
+	    }
+	}
+	break;
+#endif
     }
 }
 
@@ -295,6 +342,21 @@ sendPADI(PPPoEConnection *conn)
 	plen += sizeof(pid) + TAG_HDR_SIZE;
     }
 
+#ifdef PLUGIN
+    /* Add our maximum MTU/MRU */
+    if (MIN(lcp_allowoptions[0].mru, lcp_wantoptions[0].mru) > ETH_PPPOE_MTU) {
+	PPPoETag maxPayload;
+	UINT16_t mru = htons(MIN(lcp_allowoptions[0].mru, lcp_wantoptions[0].mru));
+	maxPayload.type = htons(TAG_PPP_MAX_PAYLOAD);
+	maxPayload.length = htons(sizeof(mru));
+	memcpy(maxPayload.payload, &mru, sizeof(mru));
+	CHECK_ROOM(cursor, packet.payload, sizeof(mru) + TAG_HDR_SIZE);
+	memcpy(cursor, &maxPayload, sizeof(mru) + TAG_HDR_SIZE);
+	cursor += sizeof(mru) + TAG_HDR_SIZE;
+	plen += sizeof(mru) + TAG_HDR_SIZE;
+    }
+#endif
+
     packet.length = htons(plen);
 
     sendPacket(conn, conn->discoverySocket, &packet, (int) (plen + HDR_SIZE));
@@ -331,10 +393,9 @@ waitForPADO(PPPoEConnection *conn, int timeout)
 
     struct PacketCriteria pc;
     pc.conn          = conn;
-    pc.acNameOK      = (conn->acName)      ? 0 : 1;
-    pc.serviceNameOK = (conn->serviceName) ? 0 : 1;
-    pc.seenACName    = 0;
-    pc.seenServiceName = 0;
+#ifdef PLUGIN
+    conn->seenMaxPayload = 0;
+#endif
 
     if (gettimeofday(&expire_at, NULL) < 0) {
 	fatalSys("gettimeofday (waitForPADO)");
@@ -408,7 +469,24 @@ waitForPADO(PPPoEConnection *conn, int timeout)
 		printErr("Ignoring PADO packet from non-unicast MAC address");
 		continue;
 	    }
+#ifdef PLUGIN
+	    if (conn->req_peer
+		&& memcmp(packet.ethHdr.h_source, conn->req_peer_mac, ETH_ALEN) != 0) {
+		warn("Ignoring PADO packet from wrong MAC address");
+		continue;
+	    }
+#endif
+	    pc.gotError = 0;
+	    pc.seenACName    = 0;
+	    pc.seenServiceName = 0;
+	    pc.acNameOK      = (conn->acName)      ? 0 : 1;
+	    pc.serviceNameOK = (conn->serviceName) ? 0 : 1;
 	    parsePacket(&packet, parsePADOTags, &pc);
+	    if (pc.gotError) {
+		printErr("Error in PADO packet");
+		continue;
+	    }
+
 	    if (!pc.seenACName) {
 		printErr("Ignoring PADO packet with no AC-Name tag");
 		continue;
@@ -509,6 +587,21 @@ sendPADR(PPPoEConnection *conn)
 	plen += ntohs(conn->relayId.length) + TAG_HDR_SIZE;
     }
 
+#ifdef PLUGIN
+    /* Add our maximum MTU/MRU */
+    if (MIN(lcp_allowoptions[0].mru, lcp_wantoptions[0].mru) > ETH_PPPOE_MTU) {
+	PPPoETag maxPayload;
+	UINT16_t mru = htons(MIN(lcp_allowoptions[0].mru, lcp_wantoptions[0].mru));
+	maxPayload.type = htons(TAG_PPP_MAX_PAYLOAD);
+	maxPayload.length = htons(sizeof(mru));
+	memcpy(maxPayload.payload, &mru, sizeof(mru));
+	CHECK_ROOM(cursor, packet.payload, sizeof(mru) + TAG_HDR_SIZE);
+	memcpy(cursor, &maxPayload, sizeof(mru) + TAG_HDR_SIZE);
+	cursor += sizeof(mru) + TAG_HDR_SIZE;
+	plen += sizeof(mru) + TAG_HDR_SIZE;
+    }
+#endif
+
     packet.length = htons(plen);
     sendPacket(conn, conn->discoverySocket, &packet, (int) (plen + HDR_SIZE));
 #ifdef DEBUGGING_ENABLED
@@ -647,8 +740,8 @@ waitForPADS(PPPoEConnection *conn, int timeout)
 void
 discovery(PPPoEConnection *conn)
 {
-    int padiAttempts = 0;
-    int padrAttempts = 0;
+    int padiAttempts;
+    int padrAttempts;
     int timeout = conn->discoveryTimeout;
 
     /* Skip discovery? */
@@ -661,6 +754,8 @@ discovery(PPPoEConnection *conn)
 	return;
     }
 
+  SEND_PADI:
+    padiAttempts = 0;
     do {
 	padiAttempts++;
 	if (padiAttempts > MAX_PADI_ATTEMPTS) {
@@ -693,6 +788,7 @@ discovery(PPPoEConnection *conn)
     }
 
     timeout = conn->discoveryTimeout;
+    padrAttempts = 0;
     do {
 	padrAttempts++;
 	if (padrAttempts > MAX_PADI_ATTEMPTS) {
@@ -700,6 +796,8 @@ discovery(PPPoEConnection *conn)
 		padrAttempts = 0;
 		timeout = conn->discoveryTimeout;
 		printErr("Timeout waiting for PADS packets");
+		/* Go back to sending PADI again */
+		goto SEND_PADI;
 	    } else {
 		rp_fatal("Timeout waiting for PADS packets");
 	    }
@@ -710,6 +808,13 @@ discovery(PPPoEConnection *conn)
 	timeout *= 2;
     } while (conn->discoveryState == STATE_SENT_PADR);
 
+#ifdef PLUGIN
+    if (!conn->seenMaxPayload) {
+	/* RFC 4638: MUST limit MTU/MRU to 1492 */
+	if (lcp_allowoptions[0].mru > ETH_PPPOE_MTU) lcp_allowoptions[0].mru = ETH_PPPOE_MTU;
+	if (lcp_wantoptions[0].mru > ETH_PPPOE_MTU)  lcp_wantoptions[0].mru = ETH_PPPOE_MTU;
+    }
+#endif
     /* We're done. */
     conn->discoveryState = STATE_SESSION;
     return;
