@@ -4,7 +4,7 @@
 *
 * Implementation of user-space PPPoE redirector for Linux.
 *
-* Copyright (C) 2000-2006 by Roaring Penguin Software Inc.
+* Copyright (C) 2000-2012 by Roaring Penguin Software Inc.
 *
 * This program may be distributed according to the terms of the GNU
 * General Public License, version 2 or (at your option) any later version.
@@ -14,7 +14,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: pppoe.c,v 1.43 2006/02/23 15:40:42 dfs Exp $";
+"$Id$";
 
 #include "pppoe.h"
 
@@ -67,52 +67,9 @@ int optFloodDiscovery    = 0;   /* Flood server with discovery requests.
 				   USED FOR STRESS-TESTING ONLY.  DO NOT
 				   USE THE -F OPTION AGAINST A REAL ISP */
 
-#ifdef AUTO_IFUP
-/* for interface activation, based on stripped down source source of ifconfig*/
-#include <linux/if.h>
-/*#include <sys/socket.h> */
-#include <sys/types.h>
-#include <sys/socket.h>
-#include "config.h"
-int skfd = -1;			/* generic raw socket desc.     */
-int sockets_open(int family)
-{
-  int sfd = -1;
-  sfd = socket(AF_INET, SOCK_DGRAM, 0);
-  return sfd;
-}
-/* Like strncpy but make sure the resulting string is always 0 terminated.
- * Ripped from util.c (net-tools package) */  
-char *safe_strncpy(char *dst, const char *src, size_t size)
-{   
-    dst[size-1] = '\0';
-    return strncpy(dst,src,size-1);   
-}
-/* Set a certain interface flag. Ripped from ifconfig.c */
-static int set_flag(char *ifname, short flag)
-{
-    struct ifreq ifr;
-
-    safe_strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
-    if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) {
-	fprintf(stderr, "%s: unknown interface: %s\n", 
-		ifname,	strerror(errno));
-	return (-1);
-    }
-    safe_strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
-    ifr.ifr_flags |= flag;
-    if (ioctl(skfd, SIOCSIFFLAGS, &ifr) < 0) {
-	perror("SIOCSIFFLAGS");
-	return -1;
-    }
-    return (0);
-}
-#endif
-
 PPPoEConnection *Connection = NULL; /* Must be global -- used
 				       in signal handler */
 
-int persist = 0; 		/* We are not a pppd plugin */
 /***********************************************************************
 *%FUNCTION: sendSessionPacket
 *%ARGUMENTS:
@@ -274,9 +231,6 @@ session(PPPoEConnection *conn)
     int maxFD = 0;
     int r;
 
-    /* Open a session socket */
-    conn->sessionSocket = openInterface(conn->ifName, Eth_PPPOE_Session, conn->myEth);
-
     /* Drop privileges */
     dropPrivs();
 
@@ -451,6 +405,7 @@ main(int argc, char *argv[])
     unsigned int s;		/* Temporary to hold session */
     FILE *pidfile;
     unsigned int discoveryType, sessionType;
+    char const *options;
 
     PPPoEConnection conn;
 
@@ -476,7 +431,6 @@ main(int argc, char *argv[])
     /* Initialize syslog */
     openlog("pppoe", LOG_PID, LOG_DAEMON);
 
-    char const *options;
 #ifdef DEBUGGING_ENABLED
     options = "I:VAT:D:hS:C:Usm:np:e:kdf:F:t:";
 #else
@@ -628,16 +582,6 @@ main(int argc, char *argv[])
 #endif
     }
 
-#ifdef AUTO_IFUP    
-    /* Create a channel to the NET kernel. */
-    if ((skfd = sockets_open(0)) < 0) {
-	perror("socket");
-	exit(1);
-    }
-
-    set_flag(conn.ifName, (IFF_UP | IFF_RUNNING));
-#endif
-
     if (!conn.printACNames) {
 
 #ifdef HAVE_N_HDLC
@@ -669,6 +613,8 @@ main(int argc, char *argv[])
 	    if (conn.printACNames) {
 		fprintf(stderr, "Sending discovery flood %d\n", n+1);
 	    }
+            conn.discoverySocket =
+	        openInterface(conn.ifName, Eth_PPPOE_Discovery, conn.myEth, NULL);
 	    discovery(&conn);
 	    conn.discoveryState = STATE_SENT_PADI;
 	    close(conn.discoverySocket);
@@ -676,7 +622,22 @@ main(int argc, char *argv[])
 	exit(EXIT_SUCCESS);
     }
 
-    discovery(&conn);
+    /* Open session socket before discovery phase, to avoid losing session */
+    /* packets sent by peer just after PADS packet (noted on some Cisco    */
+    /* server equipment).                                                  */
+    /* Opening this socket just before waitForPADS in the discovery()      */
+    /* function would be more appropriate, but it would mess-up the code   */
+    if (!optSkipSession)
+        conn.sessionSocket = openInterface(conn.ifName, Eth_PPPOE_Session, conn.myEth, NULL);
+
+    /* Skip discovery and don't open discovery socket? */
+    if (conn.skipDiscovery && conn.noDiscoverySocket) {
+	conn.discoveryState = STATE_SESSION;
+    } else {
+        conn.discoverySocket =
+	    openInterface(conn.ifName, Eth_PPPOE_Discovery, conn.myEth, NULL);
+        discovery(&conn);
+    }
     if (optSkipSession) {
 	printf("%u:%02x:%02x:%02x:%02x:%02x:%02x\n",
 	       ntohs(conn.session),
