@@ -390,8 +390,8 @@ addInterface(char const *ifname,
     strncpy(i->name, ifname, IFNAMSIZ);
     i->name[IFNAMSIZ] = 0;
 
-    i->discoverySock = openInterface(ifname, Eth_PPPOE_Discovery, i->mac);
-    i->sessionSock   = openInterface(ifname, Eth_PPPOE_Session,   NULL);
+    i->discoverySock = openInterface(ifname, Eth_PPPOE_Discovery, i->mac, NULL);
+    i->sessionSock   = openInterface(ifname, Eth_PPPOE_Session,   NULL, NULL);
     i->clientOK = clientOK;
     i->acOK = acOK;
 }
@@ -955,6 +955,11 @@ relayHandlePADT(PPPoEInterface const *iface,
     SessionHash *sh;
     PPPoESession *ses;
 
+    /* Destination address must be interface's MAC address */
+    if (memcmp(packet->ethHdr.h_dest, iface->mac, ETH_ALEN)) {
+	return;
+    }
+
     sh = findSession(packet->ethHdr.h_source, packet->session);
     if (!sh) {
 	return;
